@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2024 Red Hat, Inc.
+// Copyright (C) 2013-2025 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -25,12 +25,16 @@ namespace filtering
 {
 
 bool
-has_harmless_name_change(const decl_base_sptr& f, const decl_base_sptr& s);
+has_harmless_name_change(const decl_base_sptr& f,
+			 const decl_base_sptr& s,
+			 const diff_context_sptr& ctxt);
 
 bool union_diff_has_harmless_changes(const diff *d);
 
 bool
-has_harmful_name_change(const decl_base_sptr& f, const decl_base_sptr& s);
+has_harmful_name_change(const decl_base_sptr& f,
+			const decl_base_sptr& s,
+			const diff_context_sptr& ctxt);
 
 bool
 has_harmful_name_change(const diff* dif);
@@ -111,12 +115,39 @@ has_lvalue_reference_ness_change(const diff *d);
 bool
 has_void_ptr_to_ptr_change(const diff* d);
 
+bool
+has_void_to_non_void_change(const diff* d);
+
+bool
+has_void_to_non_void_change(const diff_sptr& d);
+
 bool
 has_harmless_enum_to_int_change(const diff* d);
 
 bool
 has_benign_array_of_unknown_size_change(const diff* dif);
 
+diff_category
+has_fn_return_or_parm_harmful_change(const diff* d);
+
+diff_category
+has_var_harmful_local_change(const diff* d);
+
+diff_category
+has_var_harmful_local_change(const diff_sptr& d);
+
+bool
+has_fn_with_virtual_offset_change(const diff* d);
+
+bool
+has_fn_with_virtual_offset_change(const diff_sptr& d);
+
+bool
+has_incompatible_fn_or_var_change(const diff* d);
+
+bool
+has_incompatible_fn_or_var_change(const diff_sptr& d);
+
 struct filter_base;
 /// Convenience typedef for a shared pointer to filter_base
 typedef shared_ptr<filter_base> filter_base_sptr;
