@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2024 Red Hat, Inc.
+// Copyright (C) 2013-2025 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -506,6 +506,9 @@ get_default_harmless_categories_bitmap();
 diff_category
 get_default_harmful_categories_bitmap();
 
+bool
+is_harmful_category(diff_category);
+
 ostream&
 operator<<(ostream& o, diff_category);
 
@@ -1093,6 +1096,9 @@ public:
   bool
   is_suppressed(bool &is_private_type) const;
 
+  bool
+  is_categorized_as_suppressed() const;
+
   bool
   to_be_reported() const;
 
@@ -2535,10 +2541,16 @@ public:
   added_functions();
 
   const string_function_decl_diff_sptr_map&
-  changed_functions();
+  changed_functions() const;
 
   const function_decl_diff_sptrs_type&
-  changed_functions_sorted();
+  changed_functions_sorted() const;
+
+  const function_decl_diff_sptrs_type&
+  incompatible_changed_functions() const;
+
+  function_decl_diff_sptrs_type&
+  incompatible_changed_functions();
 
   const string_var_ptr_map&
   deleted_variables() const;
@@ -2552,6 +2564,12 @@ public:
   const var_diff_sptrs_type&
   changed_variables_sorted();
 
+  const var_diff_sptrs_type&
+  incompatible_changed_variables() const;
+
+  var_diff_sptrs_type&
+  incompatible_changed_variables();
+
   const string_elf_symbol_map&
   deleted_unrefed_function_symbols() const;
 
@@ -2690,8 +2708,22 @@ public:
   size_t num_func_with_virtual_offset_changes() const;
   void num_func_with_virtual_offset_changes(size_t);
 
+  size_t num_func_with_local_harmful_changes() const;
+  void num_func_with_local_harmful_changes(size_t);
+
+  size_t num_func_with_incompatible_changes() const;
+  void num_func_with_incompatible_changes(size_t);
+
+  size_t num_var_with_local_harmful_changes() const;
+  void num_var_with_local_harmful_changes(size_t);
+
+  size_t num_var_with_incompatible_changes() const;
+  void num_var_with_incompatible_changes(size_t);
+
   size_t net_num_func_changed() const;
 
+  size_t net_num_non_incompatible_func_changed() const;
+
   size_t num_vars_removed() const;
   void num_vars_removed(size_t);
 
@@ -2716,6 +2748,8 @@ public:
 
   size_t net_num_vars_changed() const;
 
+  size_t net_num_non_incompatible_var_changed() const;
+
   size_t num_func_syms_removed() const;
   void num_func_syms_removed(size_t);
 
@@ -2926,6 +2960,9 @@ void
 print_diff_tree(corpus_diff_sptr diff_tree,
 		std::ostream&);
 
+void
+print_category(diff_category c);
+
 void
 categorize_redundancy(diff* diff_tree);
 
