@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2024 Red Hat, Inc.
+// Copyright (C) 2013-2025 Red Hat, Inc.
 
 ///@file
 
@@ -78,6 +78,8 @@ void get_comma_separated_args_of_option(const string& input_str,
 bool get_dsos_provided_by_rpm(const string& rpm_path,
 			      set<string>& provided_dsos);
 string trim_white_space(const string&);
+bool remove_white_spaces(string&);
+bool normalize_litterals(string&);
 string trim_leading_string(const string& from, const string& to_trim);
 void convert_char_stars_to_char_star_stars(const vector<char*>&,
 					   vector<char**>&);
@@ -233,7 +235,22 @@ enum file_type
   FILE_TYPE_DIR,
   /// A tar archive.  The archive can be compressed with the popular
   /// compression schemes recognized by GNU tar.
-  FILE_TYPE_TAR
+  FILE_TYPE_TAR,
+
+  // All non-tared compression scheme go under here.  When one of
+  // these is returned, the goal is to look into the uncompressed
+  // stream to get what format has been compressed, then return an
+  // enumerator for that compressed format instead.
+  //
+  // Please note that each time a new enumerator is added here, one
+  // needs to add a corresponding enumerator to the @ref
+  // compression_kind enum in abg-tools-utils.cc and update the
+  // is_compressed_file_type and get_compressed_streambuf functions
+  // accordingly.
+
+  /// The XZ (lzma) compresson scheme.
+
+  FILE_TYPE_XZ
 };
 
 /// Exit status for abidiff and abicompat tools.
@@ -370,6 +387,49 @@ create_best_elf_based_reader(const string& elf_file_path,
 			     bool show_all_types,
 			     bool linux_kernel_mode = false);
 
+/// This is a custom std::streambuf that knows how to decompress an
+/// input stream that was compressed using xz.
+///
+/// The code was inspired by the example in the source code of the xz
+/// project at
+/// https://github.com/tukaani-project/xz/blob/master/doc/examples/02_decompress.c.
+///
+/// here is an example of how a user code would use this custom
+/// streambuf to decode an xz'ed file and emit its content to stdout.
+///
+///        ifstream input_file("/path/to/a/compressed/file.xz", ifstream::binary);
+///        xz_decompressor_type xzed_streambuf(input_file);
+///        istream input_stream(&xzed_streambuf);
+///
+///        const size_t BUFFER_SIZE = 1024 * 4;
+///        vector<char> decompressed_data(BUFFER_SIZE);
+///        input_stream.read(decompressed_data.data(), BUFFER_SIZE);
+///        size_t nb_bytes_read = input_stream.gcount();
+///        while (nb_bytes_read && !input_stream.bad())
+///          {
+///            for (auto c : decompressed_data)
+///            std::out << c;
+///          }
+///        input_file.close();
+///
+/// Voila.
+class xz_decompressor_type : public std::streambuf
+{
+  struct priv;
+
+  std::unique_ptr<priv> priv_;
+
+  public:
+  xz_decompressor_type(std::istream& xz_istream);
+
+  ~xz_decompressor_type();
+
+  protected:
+
+  int_type
+  underflow() override;
+}; // end class xz_decompressor_type.
+
 }// end namespace tools_utils
 
 /// A macro that expands to aborting the program when executed.
