@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2024 Red Hat, Inc.
+// Copyright (C) 2013-2025 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -34,12 +34,22 @@ has_offset_changes(const string_decl_base_sptr_map& f_data_members,
 		   const string_decl_base_sptr_map& s_data_members);
 
 static bool
-type_diff_has_cv_qual_change_only(const diff *type_dif);
+type_diff_has_typedef_cv_qual_change_only(const diff *type_dif);
 
 static bool
-type_diff_has_cv_qual_change_only(const type_base_sptr& f,
-				  const type_base_sptr& s);
+type_diff_has_typedef_cv_qual_change_only(const type_base_sptr& f,
+					  const type_base_sptr& s);
 
+static diff_category
+has_harmful_change(const diff* d);
+
+static bool
+has_harmful_enum_change(const diff* diff);
+
+static bool
+has_harmless_enum_change(const type_base_sptr& f,
+			 const type_base_sptr& s,
+			 const diff_context_sptr& ctxt);
 using std::dynamic_pointer_cast;
 
 /// Walk the diff sub-trees of a a @ref corpus_diff and apply a filter
@@ -323,7 +333,7 @@ has_offset_changes(const string_decl_base_sptr_map& f_data_members,
 
 /// Test if the local changes of a @ref class_diff are harmless.
 ///
-/// Potentially harmful changes are basically:
+/// Harmful changes are basically:
 ///   1/ name change (that changes the type altogether)
 ///   2/ size change
 ///   3/ offset change of any data member
@@ -363,7 +373,7 @@ class_diff_has_only_harmless_changes(const class_diff* d)
 
 /// Test if the local changes of a @ref class_diff are harmless.
 ///
-/// Potentially harmful changes are basically:
+/// Harmful changes are basically:
 ///   1/ name change (that changes the type altogether)
 ///   2/ size change
 ///   3/ offset change of any data member
@@ -629,6 +639,24 @@ is_compatible_change(const decl_base_sptr& d1, const decl_base_sptr& d2)
   return false;
 }
 
+/// Test if a diff node carries a compatible type change.
+///
+/// @param d the diff node to consider.
+///
+/// @return true iff @p carries a compatible type change.
+static bool
+is_compatible_type_change(const diff* d)
+{
+  if (!d)
+    return false;
+
+  if (type_base_sptr t1 = is_type(d->first_subject()))
+    if (type_base_sptr t2 = is_type(d->second_subject()))
+      return types_are_compatible(t1, t2);
+
+  return false;
+}
+
 /// Test if a diff node carries a non-compatible change between two
 /// types of different kinds.
 ///
@@ -646,7 +674,7 @@ is_non_compatible_distinct_change(const diff *d)
   if (const distinct_diff* dd = is_distinct_diff(d))
     {
       if (dd->compatible_child_diff()
-	  || type_diff_has_cv_qual_change_only(d)
+	  || is_compatible_type_change(d)
 	  || (!dd->first_subject() || !dd->second_subject()))
 	// The distinct diff node carries a compatible or benign
 	// change
@@ -660,46 +688,6 @@ is_non_compatible_distinct_change(const diff *d)
   return false;
 }
 
-/// Test if two decls have different names.
-///
-/// @param d1 the first declaration to consider.
-///
-/// @param d2 the second declaration to consider.
-///
-/// @return true if d1 and d2 have different names.
-static bool
-decl_name_changed(const type_or_decl_base* a1, const type_or_decl_base *a2)
-{
-  string d1_name, d2_name;
-
-  const decl_base *d1 = dynamic_cast<const decl_base*>(a1);
-  if (d1 == 0)
-    return false;
-
-  const decl_base *d2 = dynamic_cast<const decl_base*>(a2);
-  if (d2 == 0)
-    return false;
-
-  if (d1)
-    d1_name = d1->get_qualified_name();
-  if (d2)
-    d2_name = d2->get_qualified_name();
-
-  return d1_name != d2_name;
-}
-
-/// Test if two decls have different names.
-///
-/// @param d1 the first declaration to consider.
-///
-/// @param d2 the second declaration to consider.
-///
-/// @return true if d1 and d2 have different names.
-static bool
-decl_name_changed(const type_or_decl_base_sptr& d1,
-		  const type_or_decl_base_sptr& d2)
-{return decl_name_changed(d1.get(), d2.get());}
-
 /// Test if a diff node carries a changes in which two decls have
 /// different names.
 ///
@@ -711,42 +699,6 @@ static bool
 decl_name_changed(const diff *d)
 {return decl_name_changed(d->first_subject(), d->second_subject());}
 
-/// Test if a diff node carries a change whereby two integral types
-/// have different names in a harmless way.
-///
-/// Basically, if the integral type name change is accompanied by a
-/// size change then the change is considered harmful.  If there are
-/// modifiers change, the change is considered harmful.
-static bool
-integral_type_has_harmless_name_change(const decl_base_sptr& f,
-				       const decl_base_sptr& s)
-{
-  if ((is_integral_type(f) || f->get_name().empty())
-      && (is_integral_type(s) || s->get_name().empty())
-      && decl_name_changed(f, s)
-      && (is_type(f)->get_size_in_bits()
-	  == is_type(s)->get_size_in_bits())
-      && (is_type(f)->get_alignment_in_bits()
-	  == is_type(s)->get_alignment_in_bits()))
-    {
-      real_type fi, si;
-      ABG_ASSERT(f->get_name().empty()
-		 || parse_real_type(f->get_name(), fi));
-      ABG_ASSERT(s->get_name().empty()
-		 || parse_real_type(s->get_name(), si));
-
-      if (fi.get_base_type() == si.get_base_type()
-	  && fi.get_modifiers() != si.get_modifiers())
-	// The base type hasn't changed.  That means only modifiers
-	// changed.  This is considered has harmful by default.
-	return false;
-
-      return true;
-    }
-
-  return false;
-}
-
 /// Test if two decls represents a harmless name change.
 ///
 /// For now, a harmless name change is considered only for a typedef,
@@ -756,9 +708,14 @@ integral_type_has_harmless_name_change(const decl_base_sptr& f,
 ///
 /// @param s the second decl to consider in the comparison.
 ///
+/// @param ctxt the diff context to use for fine grained comparison of
+/// @p f and @p s.
+///
 /// @return true iff decl @p s represents a harmless change over @p f.
 bool
-has_harmless_name_change(const decl_base_sptr& f, const decl_base_sptr& s)
+has_harmless_name_change(const decl_base_sptr& f,
+			 const decl_base_sptr& s,
+			 const diff_context_sptr& ctxt)
 {
   // So, a harmless name change is either ...
   return (decl_name_changed(f, s)
@@ -772,36 +729,21 @@ has_harmless_name_change(const decl_base_sptr& f, const decl_base_sptr& s)
 		&& s->get_is_anonymous_or_has_anonymous_parent())
 	       && tools_utils::decl_names_equal(f->get_qualified_name(),
 						s->get_qualified_name()))
-	      // ... a typedef name change, without having the
-	      // underlying type changed ...
-	      || (is_typedef(f)
-		  && is_typedef(s)
-		  && (is_typedef(f)->get_underlying_type()
-		   == is_typedef(s)->get_underlying_type()))
-	      // ... Types are compatible (equal modulo a typedef) ...
+	      // ... Types are compatible (equal modulo a typedef or
+	      // cv quals) ...
 	      || (is_type(f)
 		  && is_type(s)
 		  && types_are_compatible(is_type(f), is_type(s)))
-	      // ... Only qualifers changed on the type without having
-	      // the underlying type changed ...
-	      || type_diff_has_cv_qual_change_only(is_type(f), is_type(s))
+	      || has_harmless_enum_change(is_type(f), is_type(s), ctxt)
 	      // ... or a data member name change, without having its
 	      // type changed ...
 	      || (is_data_member(f)
 		  && is_data_member(s)
 		  && (is_var_decl(f)->get_type()
-		      == is_var_decl(s)->get_type()))
-	      // .. an enum name change without having any other part
-	      // of the enum to change.
-	      || (is_enum_type(f)
-		  && is_enum_type(s)
-		  && !enum_has_non_name_change(*is_enum_type(f),
-					       *is_enum_type(s),
-					       0))
-	      || integral_type_has_harmless_name_change(f, s)));
+		      == is_var_decl(s)->get_type()))));
 }
 
-/// Test if two decls represents a harmful name change.
+/// Test if two decls represent a harmful name change.
 ///
 /// A harmful name change is a name change that is not harmless, so
 /// this function uses the function has_harmless_name_change.
@@ -810,11 +752,15 @@ has_harmless_name_change(const decl_base_sptr& f, const decl_base_sptr& s)
 ///
 /// @param s the second decl to consider in the comparison.
 ///
+/// @param ctxt the diff context to use for comparison.
+///
 /// @return true iff decl @p s represents a harmful name change over
 /// @p f.
 bool
-has_harmful_name_change(const decl_base_sptr& f, const decl_base_sptr& s)
-{return decl_name_changed(f, s) && ! has_harmless_name_change(f, s);}
+has_harmful_name_change(const decl_base_sptr& f,
+			const decl_base_sptr& s,
+			const diff_context_sptr& ctxt)
+{return decl_name_changed(f, s) && ! has_harmless_name_change(f, s, ctxt);}
 
 /// Test if a diff node represents a harmful name change.
 ///
@@ -833,7 +779,7 @@ has_harmful_name_change(const diff* dif)
   decl_base_sptr f = is_decl(dif->first_subject()),
     s = is_decl(dif->second_subject());
 
-  return has_harmful_name_change(f, s);
+  return has_harmful_name_change(f, s, dif->context());
 }
 
 /// Test if a class_diff node has non-static members added or
@@ -1699,22 +1645,52 @@ has_enumerator_insertion(const diff* diff)
   return false;
 }
 
-/// Test if an enum_diff carries an enumerator removal.
+/// Test if an enum_diff carries an enumerator removal or an
+/// enumerator value change.
 ///
 /// @param diff the enum_diff to consider.
 ///
 /// @return true iff @p diff carries an enumerator removal or change.
 static bool
-has_enumerator_removal_or_change(const diff* diff)
+has_enumerator_removal_or_value_change(const diff* diff)
 {
   if (const enum_diff* d = dynamic_cast<const enum_diff*>(diff))
-    return (!d->deleted_enumerators().empty()
-	    || !d->changed_enumerators().empty());
+    {
+      if (!d->deleted_enumerators().empty())
+	return true;
+
+      for (auto& entry : d->changed_enumerators())
+	{
+	  const changed_enumerator& change = entry.second;
+	  if (change.first.get_value() != change.second.get_value())
+	    return true;
+	}
+    }
+  return false;
+}
+
+/// Test if a diff node carries an enumerator name or value change.
+///
+/// @param diff the diff node to consider.
+///
+/// @return true iff the diff node @p diff carries an enumerator name
+/// or value change.
+static bool
+has_enumerator_change(const diff* diff)
+{
+  if (const enum_diff* d = dynamic_cast<const enum_diff*>(diff))
+    return !d->changed_enumerators().empty();
   return false;
 }
 
 /// Test if an enum_diff carries a harmful change.
 ///
+/// For now, a harmful enum change is either a change that:
+///
+///   - changes the size of the enum type
+///
+///   - or removes (or changes) an existing enumerator value.
+///
 /// @param diff the enum_diff to consider.
 ///
 /// @return true iff @p diff carries a harmful change.
@@ -1722,8 +1698,9 @@ static bool
 has_harmful_enum_change(const diff* diff)
 {
   if (const enum_diff* d = dynamic_cast<const enum_diff*>(diff))
-    return (has_enumerator_removal_or_change(d)
-	    || has_type_size_change(d));
+    if (has_type_size_change(d) || has_enumerator_removal_or_value_change(d))
+      return true;
+
   return false;
 }
 
@@ -1773,6 +1750,77 @@ has_harmless_enum_to_int_change(const diff* diff)
   return false;
 }
 
+/// Test if two types represent a harmless (that can be filtered out
+/// by default) enum type change.
+///
+/// A harmless enum type change is either an enumerator insertion or
+/// an enumerator change that doesn't represents a harmful enum change
+/// at the same time.  Note that a harmless enum to int change is a
+/// harmless enum change too.
+///
+/// @param t1 the first version of the type to consider.
+///
+/// @param t2 the second version of the type to consider.
+///
+/// @param ctxt the diff context to use to compare @p t1 and @p t2.
+///
+/// @return true iff {t1, t2} represents a harmless enum change.
+static bool
+has_harmless_enum_change(const type_base_sptr& t1,
+			 const type_base_sptr& t2,
+			 const diff_context_sptr& ctxt)
+{
+  type_base_sptr f = peel_typedef_type(t1);
+  type_base_sptr s = peel_typedef_type(t2);
+  enum_type_decl_sptr e1 = is_enum_type(f);
+  enum_type_decl_sptr e2 = is_enum_type(s);
+
+  if (!e1 || !e2)
+    return false;
+
+  enum_diff_sptr dyf = compute_diff(e1, e2, ctxt);
+  if (((has_enumerator_insertion(dyf.get()) || has_enumerator_change(dyf.get()))
+       && !has_harmful_enum_change(dyf.get()))
+      || has_harmless_enum_to_int_change(dyf.get()))
+    return true;
+
+  return false;
+}
+
+/// Test if two types represent a harmless (that can be filtered out
+/// by default) enum type change.
+///
+/// A harmless enum type change is either an enumerator insertion or
+/// an enumerator change that doesn't represents a harmful enum change
+/// at the same time.  Note that a harmless enum to int change is a
+/// harmless enum change too.
+///
+/// @param t1 the first version of the type to consider.
+///
+/// @param t2 the second version of the type to consider.
+///
+/// @param ctxt the diff context to use to compare @p t1 and @p t2.
+///
+/// @return true iff {t1, t2} represents a harmless enum change.
+static bool
+has_harmless_enum_change(const diff* d)
+{
+  if (!d)
+    return false;
+
+  if (((has_enumerator_insertion(d) || has_enumerator_change(d))
+       && !has_harmful_enum_change(d))
+      || has_harmless_enum_to_int_change(d))
+    return true;
+
+  type_base_sptr f = is_type(d->first_subject());
+  type_base_sptr s = is_type(d->second_subject());
+  if (!f || !s)
+    return false;
+
+  return has_harmless_enum_change(f, s, d->context());
+}
+
 /// Test if an @ref fn_parm_diff node has a top cv qualifier change on
 /// the type of the function parameter.
 ///
@@ -1839,7 +1887,7 @@ has_fn_parm_type_top_cv_qual_change(const diff* diff)
 ///
 /// @return true iff the type_diff carries a CV qualifier only change.
 static bool
-type_diff_has_cv_qual_change_only(const diff *type_dif)
+type_diff_has_typedef_cv_qual_change_only(const diff *type_dif)
 {
   if (!type_dif)
     return false;
@@ -1847,19 +1895,22 @@ type_diff_has_cv_qual_change_only(const diff *type_dif)
   type_base_sptr f = is_type(type_dif->first_subject());
   type_base_sptr s = is_type(type_dif->second_subject());
 
-  return type_diff_has_cv_qual_change_only(f, s);
+  return type_diff_has_typedef_cv_qual_change_only(f, s);
 }
 
 /// Test if a type only carries a CV qualifier-only change.
 ///
+/// Note that for pointers and array types, the functions look at
+/// pointed-to types for comparison.
+///
 /// @param f the first version of the type.
 ///
 /// @param s the second version of the type.
 ///
 /// @return true iff the change is only a qualifier change.
 static bool
-type_diff_has_cv_qual_change_only(const type_base_sptr& f,
-				  const type_base_sptr& s)
+type_diff_has_typedef_cv_qual_change_only(const type_base_sptr& f,
+					  const type_base_sptr& s)
 {
   type_base_sptr a = f;
   type_base_sptr b = s;
@@ -1871,29 +1922,17 @@ type_diff_has_cv_qual_change_only(const type_base_sptr& f,
     return true;
 
   if (is_pointer_type(a) && is_pointer_type(b))
-    {
-      a = peel_pointer_type(a);
-      b = peel_pointer_type(b);
-    }
-
-  if (a && b && *a == *b)
-    return true;
-
-  a = peel_qualified_or_typedef_type(a);
-  b = peel_qualified_or_typedef_type(b);
-
-  if (a && b && *a == *b)
-    return true;
+    return equals_modulo_cv_qualifier(is_pointer_type(a), is_pointer_type(b));
 
   // If f and s are arrays, note that they can differ only by the cv
   // qualifier of the array element type.  That cv qualifier is not
   // removed by peel_qualified_type.  So we need to test this case
   // specifically.
-  if (array_type_def *f_a = is_array_type(a.get()))
-    if (array_type_def *s_a = is_array_type(b.get()))
+  if (array_type_def_sptr f_a = is_array_type(a))
+    if (array_type_def_sptr s_a = is_array_type(b))
       return equals_modulo_cv_qualifier(f_a, s_a);
 
-  return (a && b && *a == *b);
+  return false;
 }
 
 /// Test if an @ref fn_parm_diff node has a cv qualifier change on the
@@ -1917,7 +1956,7 @@ has_fn_parm_type_cv_qual_change(const diff* dif)
     return false;
 
   const diff *type_dif = parm_diff->type_diff().get();
-  return type_diff_has_cv_qual_change_only(type_dif);
+  return type_diff_has_typedef_cv_qual_change_only(type_dif);
 }
 
 /// Test if a function type or decl diff node carries a CV
@@ -1941,7 +1980,7 @@ has_fn_return_type_cv_qual_change(const diff* dif)
     return false;
 
   const diff* return_type_diff = fn_type_diff->return_type_diff().get();
-  return type_diff_has_cv_qual_change_only(return_type_diff);
+  return type_diff_has_typedef_cv_qual_change_only(return_type_diff);
 }
 
 /// Test if a function type or decl diff node carries a function
@@ -1957,7 +1996,7 @@ static bool
 has_added_or_removed_function_parameters(const diff *dif)
 {
   const function_type_diff *fn_type_diff = is_function_type_diff(dif);
-    if (!fn_type_diff)
+  if (!fn_type_diff)
     if (const function_decl_diff* fn_decl_diff = is_function_decl_diff(dif))
       fn_type_diff = fn_decl_diff->type_diff().get();
 
@@ -1971,6 +2010,155 @@ has_added_or_removed_function_parameters(const diff *dif)
   return false;
 }
 
+/// Test if a diff node is a function diff node that carries either a
+/// return or a parameter type change that is deemed harmful.
+///
+/// @param d the diff node to consider.
+///
+/// @return the category of the change carried by @p or zero if
+/// doesn't carry any change.
+diff_category
+has_fn_return_or_parm_harmful_change(const diff* d)
+{
+  const function_decl_diff* fn_decl_diff = nullptr;
+  const function_type_diff* fn_type_diff = is_function_type_diff(d);
+
+  if (!fn_type_diff)
+    fn_decl_diff = is_function_decl_diff(d);
+
+  if (!fn_decl_diff && !fn_type_diff)
+    return NO_CHANGE_CATEGORY;
+
+  diff_category category = NO_CHANGE_CATEGORY;
+  if (fn_decl_diff)
+    category = fn_decl_diff->get_local_category();
+
+  if (is_harmful_category(category))
+    return category;
+
+  if (fn_decl_diff)
+    fn_type_diff = fn_decl_diff->type_diff().get();
+
+  if (!fn_type_diff)
+    return NO_CHANGE_CATEGORY;
+
+  diff_sptr return_type_diff = fn_type_diff->return_type_diff();
+  if (return_type_diff && !has_void_to_non_void_change(return_type_diff))
+    category = return_type_diff->get_local_category();
+
+  if (is_harmful_category(category))
+    return category;
+
+  for (const auto& entry : fn_type_diff->subtype_changed_parms())
+    {
+      category = entry.second->get_local_category();
+      if (is_harmful_category(category))
+	return category;
+    }
+
+  return NO_CHANGE_CATEGORY;
+}
+
+/// Test if a diff node carries a change to the offset of a virtual
+/// function.
+///
+/// @param d the diff node to consider.
+///
+/// @return true iff @p carries a change to the offset of a virtual
+/// function.
+bool
+has_fn_with_virtual_offset_change(const diff* d)
+{
+  const function_decl_diff* fn_diff = is_function_decl_diff(d);
+  if (!fn_diff)
+    return false;
+
+  if (fn_diff->get_local_category() & VIRTUAL_MEMBER_CHANGE_CATEGORY)
+    return true;
+
+  return false;
+}
+
+/// Test if a diff node carries a change to the offset of a virtual
+/// function.
+///
+/// @param d the diff node to consider.
+///
+/// @return true iff @p carries a change to the offset of a virtual
+/// function.
+bool
+has_fn_with_virtual_offset_change(const diff_sptr& d)
+{return has_fn_with_virtual_offset_change(d.get());}
+
+
+/// Test if a diff node carries a harmful local change to a variable.
+///
+/// @param d the diff node to consider.
+///
+/// @return the @ref diff_category of the harmful local change or zero
+/// if the diff node carries no harmful local change.
+diff_category
+has_var_harmful_local_change(const diff* d)
+{
+  const var_diff* vd = is_var_diff(d);
+  diff_category cat = NO_CHANGE_CATEGORY;
+
+  if (!vd || has_benign_array_of_unknown_size_change(d))
+    return cat;
+
+  cat = vd->get_local_category();
+  if (is_harmful_category(cat))
+    return cat;
+
+  diff_sptr type_diff = vd->type_diff();
+
+  cat = type_diff->get_local_category();
+  if (is_harmful_category(cat))
+    return cat;
+
+  return NO_CHANGE_CATEGORY;
+}
+
+/// Test if diff node carries a harmful local change to a variable.
+///
+/// @param d the diff node to consider.
+///
+/// @return the @ref diff_category of the harmful local change or zero
+/// if the diff node carries no harmful local change.
+diff_category
+has_var_harmful_local_change(const diff_sptr& d)
+{return has_var_harmful_local_change(d.get());}
+
+/// Test if a diff node carries an incompatible ABI change.
+///
+/// An incompatible ABI change is a harmful ABI change (i.e, one that
+/// cannot be filtered out) that definitely makes the new ABI
+/// incompatible with the previous one.
+///
+/// @param d the diff node to consider.
+///
+/// @return true iff @p d carries an incompatible ABI change.
+bool
+has_incompatible_fn_or_var_change(const diff* d)
+{
+  return (has_fn_return_or_parm_harmful_change(d)
+	  || has_fn_with_virtual_offset_change(d)
+	  || has_var_harmful_local_change(d));
+}
+
+/// Test if a diff node carries an incompatible ABI change.
+///
+/// An incompatible ABI change is a harmful ABI change (i.e, one that
+/// cannot be filtered out) that definitely makes the new ABI
+/// incompatible with the previous one.
+///
+/// @param d the diff node to consider.
+///
+/// @return true iff @p d carries an incompatible ABI change.
+bool
+has_incompatible_fn_or_var_change(const diff_sptr& d)
+{return has_incompatible_fn_or_var_change(d.get());}
+
 /// Test if a variable diff node carries a CV qualifier change on its type.
 ///
 /// @param dif the diff node to consider.  Note that if it's not of
@@ -1989,7 +2177,7 @@ has_var_type_cv_qual_change(const diff* dif)
   if (!type_dif)
     return false;
 
-  return type_diff_has_cv_qual_change_only(type_dif);
+  return type_diff_has_typedef_cv_qual_change_only(type_dif);
 }
 
 /// Test if a type change is a "void pointer to pointer" change.
@@ -2013,6 +2201,79 @@ is_void_ptr_to_ptr(const type_base* f, const type_base* s)
   return false;
 }
 
+/// Test if a pair of types represents a "void-to-non-void" change.
+///
+/// The test looks through potential typedefs.
+///
+/// @param f the first type to consider.
+///
+/// @param s the second type to consider.
+///
+/// @return true iff the pair of types represents a void-to-non-void
+/// type change.
+static bool
+is_void_to_non_void(const type_base* f, const type_base* s)
+{
+  f = peel_typedef_type(f);
+  s = peel_typedef_type(s);
+
+  if (!f || !s)
+    return false;
+
+  const environment& env = f->get_environment();
+  if (env.is_void_type(f) &&  !env.is_void_type(s))
+    return true;
+
+  return false;
+}
+
+/// Test if a pair of types represents a "void-to-non-void" change.
+///
+/// The test looks through potential typedefs.
+///
+/// @param f the first type to consider.
+///
+/// @param s the second type to consider.
+///
+/// @return true iff the pair of types represents a void-to-non-void
+/// type change.
+static bool
+is_void_to_non_void(const type_base_sptr& f, const type_base_sptr s)
+{return is_void_to_non_void(f.get(), s.get());}
+
+/// Test if a diff node carries a "void-to-non-void" type change
+///
+/// The test looks through potential typedefs.
+///
+/// @param f the first type to consider.
+///
+/// @param s the second type to consider.
+///
+/// @return true iff the pair of types represents a void-to-non-void
+/// type change.
+bool
+has_void_to_non_void_change(const diff* d)
+{
+  type_base_sptr f = is_type(d->first_subject());
+  type_base_sptr s = is_type(d->second_subject());
+
+  return is_void_to_non_void(f, s);
+}
+
+/// Test if a diff node carries a "void-to-non-void" type change
+///
+/// The test looks through potential typedefs.
+///
+/// @param f the first type to consider.
+///
+/// @param s the second type to consider.
+///
+/// @return true iff the pair of types represents a void-to-non-void
+/// type change.
+bool
+has_void_to_non_void_change(const diff_sptr& d)
+{return has_void_to_non_void_change(d.get());}
+
 /// Test if a diff node carries a void* to pointer type change.
 ///
 /// Note that this function looks through typedef and qualifier types
@@ -2099,9 +2360,75 @@ union_diff_has_harmless_changes(const diff *d)
   return false;
 }
 
+/// Test if a diff node carries a change that is categorized as
+/// "harmful".
+///
+/// A harmful change is a change that is not harmless.  OK, that
+/// smells bit like a tasteless tautology, but bear with me please.
+///
+/// A harmless change is a change that should be filtered out by
+/// default to avoid unnecessarily cluttering the change report.
+///
+/// A harmful change is thus a change that SHOULD NOT be filtered out
+/// by default because it CAN represent an incompatible ABI change.
+///
+/// An incompatbile ABI change is a harmful change that makes the new
+/// ABI incompatible with the previous one.
+///
+/// @return the category of the harmful changes carried by the diff
+/// node or zero if the change carries no harmful change.
+static diff_category
+has_harmful_change(const diff* d)
+{
+  diff_category category = NO_CHANGE_CATEGORY;
+  decl_base_sptr f = is_decl(d->first_subject()),
+    s = is_decl(d->second_subject());
+
+  // Detect size or offset changes as well as data member addition
+  // or removal.
+  //
+  // TODO: be more specific -- not all size changes are harmful.
+  if (!has_class_decl_only_def_change(d)
+      && !has_enum_decl_only_def_change(d)
+      && (type_size_changed(f, s)
+	  || type_has_offset_changes(f, s)
+	  || data_member_offset_changed(f, s)
+	  || non_static_data_member_type_size_changed(f, s)
+	  || non_static_data_member_added_or_removed(d)
+	  || base_classes_removed(d)
+	  || has_harmful_enum_change(d)
+	  || crc_changed(d)
+	  || namespace_changed(d)))
+    category |= SIZE_OR_OFFSET_CHANGE_CATEGORY;
+
+  if (has_virtual_mem_fn_change(d))
+    category |= VIRTUAL_MEMBER_CHANGE_CATEGORY;
+
+  if (has_lvalue_reference_ness_change(d))
+    category |= REFERENCE_LVALUENESS_CHANGE_CATEGORY;
+
+  if (has_added_or_removed_function_parameters(d))
+    category |= FN_PARM_ADD_REMOVE_CHANGE_CATEGORY;
+
+  if (is_non_compatible_distinct_change(d))
+    category |= NON_COMPATIBLE_DISTINCT_CHANGE_CATEGORY;
+
+  if (has_harmful_name_change(d))
+    category |= NON_COMPATIBLE_NAME_CHANGE_CATEGORY;
+
+  return category;
+}
+
 /// Detect if the changes carried by a given diff node are deemed
 /// harmless and do categorize the diff node accordingly.
 ///
+/// A harmless change is a change that ought to be filtered out by
+/// default from the change report.  Filtering out harmless changes is
+/// to avoid unnecessarily cluttering the change report.
+///
+/// A change is not harmless is a harmful node.  Note that harmful
+/// diff nodes are categorized by @ref categorize_harmful_diff_node.
+///
 /// @param d the diff node being visited.
 ///
 /// @param pre this is true iff the node is being visited *before* the
@@ -2132,7 +2459,7 @@ categorize_harmless_diff_node(diff *d, bool pre)
       if (is_compatible_change(f, s))
 	category |= COMPATIBLE_TYPE_CHANGE_CATEGORY;
 
-      if (has_harmless_name_change(f, s)
+      if (has_harmless_name_change(f, s, d->context())
 	  || class_diff_has_harmless_odr_violation_change(d))
 	category |= HARMLESS_DECL_NAME_CHANGE_CATEGORY;
 
@@ -2150,9 +2477,7 @@ categorize_harmless_diff_node(diff *d, bool pre)
       if (has_data_member_replaced_by_anon_dm(d))
 	category |= HARMLESS_DATA_MEMBER_CHANGE_CATEGORY;
 
-      if ((has_enumerator_insertion(d)
-	   && !has_harmful_enum_change(d))
-	  || has_harmless_enum_to_int_change(d))
+      if (has_harmless_enum_change(d))
 	category |= HARMLESS_ENUM_CHANGE_CATEGORY;
 
       if (function_name_changed_but_not_symbol(d))
@@ -2191,6 +2516,21 @@ categorize_harmless_diff_node(diff *d, bool pre)
 /// Detect if the changes carried by a given diff node are deemed
 /// harmful and do categorize the diff node accordingly.
 ///
+/// A harmful change is a change that is not harmless.  OK, that
+/// smells bit like a tasteless tautology, but bear with me please.
+///
+/// A harmless change is a change that should be filtered out by
+/// default to avoid unnecessarily cluttering the change report.
+///
+/// A harmful change is thus a change that SHOULD NOT be filtered out
+/// by default because it CAN represent an incompatible ABI change.
+///
+/// An incompatbile ABI change is a harmful change that makes the new
+/// ABI incompatible with the previous one.
+///
+/// Note that harmless diff nodes are categorized by
+/// @ref categorize_harmless_diff_node.
+///
 /// @param d the diff node being visited.
 ///
 /// @param pre this is true iff the node is being visited *before* the
@@ -2207,40 +2547,7 @@ categorize_harmful_diff_node(diff *d, bool pre)
   if (pre)
     {
       diff_category category = NO_CHANGE_CATEGORY;
-      decl_base_sptr f = is_decl(d->first_subject()),
-	s = is_decl(d->second_subject());
-
-      // Detect size or offset changes as well as data member addition
-      // or removal.
-      //
-      // TODO: be more specific -- not all size changes are harmful.
-      if (!has_class_decl_only_def_change(d)
-	  && !has_enum_decl_only_def_change(d)
-	  && (type_size_changed(f, s)
-	      || type_has_offset_changes(f, s)
-	      || data_member_offset_changed(f, s)
-	      || non_static_data_member_type_size_changed(f, s)
-	      || non_static_data_member_added_or_removed(d)
-	      || base_classes_removed(d)
-	      || has_harmful_enum_change(d)
-	      || crc_changed(d)
-	      || namespace_changed(d)))
-	category |= SIZE_OR_OFFSET_CHANGE_CATEGORY;
-
-      if (has_virtual_mem_fn_change(d))
-	category |= VIRTUAL_MEMBER_CHANGE_CATEGORY;
-
-      if (has_lvalue_reference_ness_change(d))
-	category |= REFERENCE_LVALUENESS_CHANGE_CATEGORY;
-
-      if (has_added_or_removed_function_parameters(d))
-	category |= FN_PARM_ADD_REMOVE_CHANGE_CATEGORY;
-
-      if (is_non_compatible_distinct_change(d))
-	category |= NON_COMPATIBLE_DISTINCT_CHANGE_CATEGORY;
-
-      if (has_harmful_name_change(d))
-	category |= NON_COMPATIBLE_NAME_CHANGE_CATEGORY;
+      category = has_harmful_change(d);
 
       if (category)
 	{
