@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2021-2024 Oracle, Inc.
+// Copyright (C) 2021-2025 Oracle, Inc.
 //
 // Author: Jose E. Marchesi
 
@@ -266,7 +266,7 @@ public:
     : elf_based_reader(elf_path, debug_info_root_paths, env),
       ctfa(), ctf_sect(), symtab_sect(), strtab_sect()
   {
-    initialize();
+    reset();
   }
 
   /// Initializer of the reader.
@@ -282,7 +282,7 @@ public:
   /// the context uses resources that are allocated in the
   /// environment.
   void
-  initialize()
+  reset()
   {
     types_to_canonicalize.clear();
     cur_tu_.reset();
@@ -315,7 +315,7 @@ public:
              bool			load_all_types = false,
              bool			linux_kernel_mode = false)
   {
-    initialize();
+    reset();
     load_all_types = load_all_types;
     linux_kernel_mode = linux_kernel_mode;
     elf_based_reader::initialize(elf_path, debug_info_root_paths);
