@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2013-2024 Red Hat, Inc.
+// Copyright (C) 2013-2025 Red Hat, Inc.
 //
 // Author: Dodji Seketeli
 
@@ -46,6 +46,24 @@ static InOutSpec in_out_specs[] =
     "output/test-read-dwarf/test0.abi",
     NULL,
   },
+  {
+    "data/test-read-dwarf/test0.xz",
+    "",
+    "",
+    SEQUENCE_TYPE_ID_STYLE,
+    "data/test-read-dwarf/test0.abi",
+    "output/test-read-dwarf/test0-xz.abi",
+    NULL,
+  },
+  {
+    "data/test-read-dwarf/test0.xzbinary",
+    "",
+    "",
+    SEQUENCE_TYPE_ID_STYLE,
+    "data/test-read-dwarf/test0.abi",
+    "output/test-read-dwarf/test0-xzbinary.abi",
+    NULL,
+  },
   {
     "data/test-read-dwarf/test0",
     "",
